@@ -155,15 +155,6 @@
   (:emitter
    (emit-byte segment byte)))
 
-(define-instruction word (segment word)
-  (:emitter
-   (etypecase word
-     (fixup
-      (note-fixup segment :absolute word)
-      (emit-word segment 0))
-     (integer
-      (emit-word segment word)))))
-
 (define-instruction dword (segment word)
   (:emitter
    (etypecase word
@@ -2328,7 +2319,8 @@
                                                3))))))
             (one-instruction-emitter (segment position)
               (emit-ldr-literal segment
-                                #b01 0
+                                (if (sc-is dest 32-bit-reg) #b00 #b01)
+                                0
                                 (ldb (byte 19 0)
                                      (ash (compute-delta position) -2))
                                 (tn-offset dest)))
@@ -2628,8 +2620,9 @@
     (destructuring-bind (type value) constant
       (ecase type
         ((:byte :word :dword :qword)
-         (aver (integerp value))
-         (cons type value))
+         (etypecase value
+           ((or integer (cons (eql :layout-id)))
+            (cons type value))))
         (:base-char
          #+sb-unicode (aver (typep value 'base-char))
          (cons :byte (char-code value)))
@@ -2669,6 +2662,9 @@
 (defun size-nbyte (size)
   (ecase size
     (:byte  1)
+    ;; These keywords are completely wrong for AARCH64 but I don't want to touch them.
+    ;; The correct definitions would have :HWORD (halfword) for 2 bytes, :WORD for 4,
+    ;; :DWORD for 8, and :QWORD for 16.
     (:word  2)
     (:dword 4)
     ((:qword :fixup) 8)
@@ -2678,20 +2674,46 @@
   (stable-sort constants #'> :key (lambda (constant)
                                     (size-nbyte (caar constant)))))
 
+(sb-assem::%def-inst-encoder
+ '.layout-id
+ (lambda (segment layout)
+   (sb-c:note-fixup segment :layout-id (sb-c:make-fixup layout :layout-id))
+   (sb-assem::%emit-skip segment 4)))
+
 (defun emit-inline-constant (section constant label)
   (let* ((type (car constant))
+         (val (cdr constant))
          (size (size-nbyte type)))
     (emit section
           `(.align ,(integer-length (1- size)))
           label
-          (let ((val (cdr constant)))
-            (case type
-              (:fixup
-               ;; Use the WORD emitter which knows how to emit fixups
-               `(word ,(apply #'make-fixup val)))
-              (t
+          (cond ((typep val '(cons (eql :layout-id)))
+                 `(.layout-id ,(cadr val)))
+                ((eql type :fixup)
+                 ;; Use the DWORD emitter which knows how to emit fixups
+                 `(dword ,(apply #'make-fixup val)))
+                (t
                ;; Could add pseudo-ops for .WORD, .INT, .QUAD, .OCTA just like gcc has.
                ;; But it works fine to emit as a sequence of bytes
+               ;; FIXME: missing support for big-endian. Do we care?
                `(.byte ,@(loop repeat size
                                collect (prog1 (ldb (byte 8 0) val)
-                                         (setf val (ash val -8)))))))))))
+                                         (setf val (ash val -8))))))))))
+
+(defun sb-vm:fixup-code-object (code offset value kind flavor)
+  (declare (type index offset) (ignore flavor))
+  (unless (zerop (rem offset sb-assem:+inst-alignment-bytes+))
+    (error "Unaligned instruction?  offset=#x~X." offset))
+  (let ((sap (code-instructions code)))
+    (ecase kind
+      (:absolute
+       (setf (sap-ref-word sap offset) value))
+      (:layout-id
+       (setf (signed-sap-ref-32 sap offset) value))
+      (:cond-branch
+       (setf (ldb (byte 19 5) (sap-ref-32 sap offset))
+             (ash (- value (+ (sap-int sap) offset)) -2)))
+      (:uncond-branch
+       (setf (ldb (byte 26 0) (sap-ref-32 sap offset))
+             (ash (- value (+ (sap-int sap) offset)) -2)))))
+  nil)
